/*-----------------------------------------------------------------------*/
/* Low level disk I/O module skeleton for FatFs     (C)ChaN, 2014        */
/*-----------------------------------------------------------------------*/
/* If a working storage control module is available, it should be        */
/* attached to the FatFs via a glue function rather than modifying it.   */
/* This is an example of glue functions to attach various exsisting      */
/* storage control modules to the FatFs module with a defined API.       */
/*-----------------------------------------------------------------------*/

#include "diskio.h"		/* FatFs lower layer API */

#include "SPI_FLASH.h" //by jkl
#include "debug.h"

/* Definitions of physical drive number for each drive */
#define ATA		0	/* Example: Map ATA harddisk to physical drive 0 */
#define MMC		2	/* Example: Map MMC/SD card to physical drive 1 */
#define USB		3	/* Example: Map USB MSD to physical drive 2 */

#define SPI_FLASH 1 // ⲿ SPI Flash

/*-----------------------------------------------------------------------*/
/* Get Drive Status                                                      */
/* change by jkl  Modify the function and add external FLASH                                                      */
/*-----------------------------------------------------------------------*/

DSTATUS disk_status (
	BYTE pdrv		/* Physical drive nmuber to identify the drive */
)
{
	DSTATUS stat = STA_NOINIT;

	switch (pdrv)
	{
        case ATA :
            //result = ATA_disk_status();
            // translate the reslut code here
            return stat;
        case SPI_FLASH :
             /* SPI Flash Status detection: read SPI Flash equipment ID */
            if (W25Q32_FLASH_ID1 == FLASH_ReadID())
            {
                /* equipment ID The reading result is correct */
                stat &= ~STA_NOINIT;
                
            }
            else
            {
                /* equipment ID Reading result error */
                stat = STA_NOINIT;
              
            }
            return stat;

        case MMC :

            //result = MMC_disk_status();

            // translate the reslut code here

            return stat;

        case USB :
            //result = USB_disk_status();

            // translate the reslut code here

            return stat;
    }
	return stat;
}



/*-----------------------------------------------------------------------*/
/* Inidialize a Drive                                                    */
/*-----------------------------------------------------------------------*/

DSTATUS disk_initialize (
	BYTE pdrv				/* Physical drive nmuber to identify the drive */
)
{
	DSTATUS stat = STA_NOINIT;
    UINT i;

	switch (pdrv) {
	case ATA :
		//result = ATA_disk_initialize();

		// translate the reslut code here

		return stat;
    
    case SPI_FLASH :
            
         /* initialization SPI Flash */
         FLASH_Port_Init();
         /* delay for a short time */
         i=500;
         while (--i);
         /* wake SPI Flash */
         FLASH_WAKEUP();
         /* Get SPI Flash Chip status */
         stat=disk_status(SPI_FLASH);
        
         return stat;


	case MMC :
		//result = MMC_disk_initialize();

		// translate    the reslut code here

		return stat;

	case USB :
		//result = USB_disk_initialize();

		// translate the reslut code here

		return stat;
	}
	return stat;
}



/*-----------------------------------------------------------------------*/
/* Read Sector(s)                                                        */
/*-----------------------------------------------------------------------*/

DRESULT disk_read (
	BYTE pdrv,		/* Physical drive nmuber to identify the drive */
	BYTE *buff,		/* Data buffer to store read data */
	DWORD sector,	/* Sector address in LBA */
	UINT count		/* Number of sectors to read */
)
{
	DRESULT res = RES_PARERR;

	switch (pdrv) {
	case ATA :
		// translate the arguments here

		//result = ATA_disk_read(buff, sector, count);

		// translate the reslut code here

		return res;
    case SPI_FLASH:
        // Shifting left by 12 is equivalent to *4096
        FLASH_Read(buff, sector <<12, count<<12);
        res = RES_OK;
        break;

	case MMC :
		// translate the arguments here

		//result = MMC_disk_read(buff, sector, count);

		// translate the reslut code here

		return res;

	case USB :
		// translate the arguments here

		//result = USB_disk_read(buff, sector, count);

		// translate the reslut code here

		return res;
	}

	return res;
}



/*-----------------------------------------------------------------------*/
/* Write Sector(s)                                                       */
/*-----------------------------------------------------------------------*/

#if _USE_WRITE
DRESULT disk_write (
	BYTE pdrv,			/* Physical drive nmuber to identify the drive */
	const BYTE *buff,	/* Data to be written */
	DWORD sector,		/* Sector address in LBA */
	UINT count			/* Number of sectors to write */
)
{
	DRESULT res = RES_PARERR;
    UINT write_addr;

	switch (pdrv) {
	case ATA :
		// translate the arguments here

		//result = ATA_disk_write(buff, sector, count);

		// translate the reslut code here

		return res;
    
    case SPI_FLASH:
        /*write sector*/
        write_addr = sector<<12;
        FLASH_Erase_Sector(write_addr);
        FLASH_Write((u8 *)buff,write_addr,count<<12);
        res = RES_OK;
//        printf("write_addr=%4x\r\n",write_addr);
        break;

	case MMC :
		// translate the arguments here

		//result = MMC_disk_write(buff, sector, count);

		// translate the reslut code here

		return res;

	case USB :
		// translate the arguments here

		//result = USB_disk_write(buff, sector, count);

		// translate the reslut code here

		return res;
	}

	return res;
}
#endif


/*-----------------------------------------------------------------------*/
/* Miscellaneous Functions                                               */
/*-----------------------------------------------------------------------*/

#if _USE_IOCTL
DRESULT disk_ioctl (
	BYTE pdrv,		/* Physical drive nmuber (0..) */
	BYTE cmd,		/* Control code */
	void *buff		/* Buffer to send/receive control data */
)
{
	DRESULT res;

	switch (pdrv) {
	case ATA :

		// Process of the command for the ATA drive

		return res;

    case SPI_FLASH:
        switch (cmd) 
        {
            /* Number of sectors: 512*4096/1024/1024=2(MB) */
            case GET_SECTOR_COUNT:
            *(DWORD * )buff = 512;
                break;
            /* sector size */
            case GET_SECTOR_SIZE :
            *(WORD * )buff = 4096;
                break;
            /* Number of sectors to be erased simultaneously */
            case GET_BLOCK_SIZE :
            *(DWORD * )buff = 1;
            break;
        }
        res = RES_OK;
    break;

	case MMC :

		// Process of the command for the MMC/SD card

		return res;

	case USB :

		// Process of the command the USB drive

		return res;
	}

	return res;
}


#if 1
DWORD get_fattime(void) {
	/* Returns the current timestamp */
	return	  ((DWORD)(2015 - 1980) << 25)	/* Year 2015 */
			| ((DWORD)1 << 21)				/* Month 1 */
			| ((DWORD)1 << 16)				/* Mday 1 */
			| ((DWORD)0 << 11)				/* Hour 0 */
			| ((DWORD)0 << 5)				  /* Min 0 */
			| ((DWORD)0 >> 1);				/* Sec 0 */
}
#endif
#endif
